==> mod_msql-0.3.tar.gz: mod_msql-0.3/src/mod_msql.c <==
/*	mod_msql.c	- 
**
**
** Copyright (c) 1995-2001  Hughes Technologies Pty Ltd.
**
** Permission to use, copy, and distribute for non-commercial purposes,
** is hereby granted without fee, providing that the above copyright
** notice appear in all copies and that both the copyright notice and this
** permission notice appear in supporting documentation.
**
** The software may be modified for your own purposes, but modified versions
** may not be distributed.
**
** This software is provided "as is" without any expressed or implied warranty.
**
*/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <time.h>
#include "ember.h"
#include "msql.h"
#include "mod_msql.h"


#define	MOD_NAME	"mod_msql"
#define SOCK_DESCR	"sock"


int doMsqlConnect(script, args)
	ember	*script;
	e_stack	*args;
{
	int	sock;
	char	*host = NULL;

	if (args)
	{
		if (emArgType(args) != EMBER_TYPE_TEXT)
		{
			eDynamicSetError(script, EMBER_TYPE_INT, 
				"Invalid parameter");
			return(0);
		}
		host = emTextArg(args);
		if (args->next)
		{
			eDynamicSetError(script, EMBER_TYPE_INT, 
				"Too many arguments");
			return(0);
		}
	}
	if ((sock = msqlConnect(host)) < 0)
	{
		eDynamicSetError(script,EMBER_TYPE_INT,msqlErrMsg);
		return(0);
	}
	eDynamicSetIntValue(script, &script->returnValue, sock);
	return(0);
}

int doMsqlClose(script, args)
	ember	*script;
	e_stack	*args;
{
	int	sock;

	sock = emIntArg(args);
	msqlClose(sock);
	eDynamicSetIntValue(script, &script->returnValue, 0);
	return(0);
}



int doMsqlSelectDB(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*dbName;
	int	sock;


	sock = emIntArg(args);
	dbName = emTextArg(args->next);
	if (msqlSelectDB(sock,dbName) < 0)
	{
		eDynamicSetError(script,EMBER_TYPE_INT,msqlErrMsg);
		eDynamicSetIntValue(script, &script->returnValue, -1);
	}
	else
	{
		eDynamicSetIntValue(script, &script->returnValue, 0);
	}
	return(0);
}




int doMsqlQuery(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*query;
	int	res,
		sock;


	sock = emIntArg(args);
	query = emTextArg(args->next);
	res = msqlQuery(sock,query);
	if (res < 0)
	{
		eDynamicSetError(script,EMBER_TYPE_INT,msqlErrMsg);
	}
	eDynamicSetIntValue(script, &script->returnValue, res);
	return(0);
}


int doMsqlStoreResult(script, args)
	ember	*script;
	e_stack	*args;
{
	m_result	*res;
	u_int		id;

	res = msqlStoreResult();
	id = eDynamicStoreObject(script, MOD_NAME, SOCK_DESCR, (void *)res);
	eDynamicSetIntValue(script, &script->returnValue, id);
	return(0);
}


int doMsqlFreeResult(script, args)
	ember	*script;
	e_stack	*args;
{
	int		id;
	m_result	*res;

	id = emIntArg(args);
	res = (m_result *)eDynamicDeleteObject(script, MOD_NAME, SOCK_DESCR, id);
	if (res)
	{
		msqlFreeResult(res);
	}
	return(0);
}



int doMsqlFetchRow(script, args)
	ember	*script;
	e_stack	*args;
{
	u_int	id;
	int	numFields,
		index;
	m_row	row;
	m_result *res;
	e_value	val;

	id = emIntArg(args);
	res = (m_result *)eDynamicFetchObject(script, MOD_NAME,SOCK_DESCR, id);
	if (!res)
	{
		eDynamicSetError(script,EMBER_TYPE_TEXT, "Bad result handle");
		return(-1);
	}

	numFields = msqlNumFields(res);
	row = msqlFetchRow(res);
	if (!row)
	{
		eDynamicSetTextValue(script, &script->returnValue, "");
		return(0);
	}
	index = 0;
	while (index < numFields)
	{
		if (row[index])
			eDynamicSetTextValue(script,&val,row[index]);
		else
			eDynamicSetTextValue(script,&val,"");
		eDynamicSetArrayElement(script, &script->returnValue, &val, index);
		index++;
	}
	return(0);
}

int doMsqlNumRows(script, args)
	ember	*script;
	e_stack	*args;
{
	int	numRows;
	u_int	id;
	m_result *res;

	id = emIntArg(args);
	res = (m_result *)eDynamicFetchObject(script, MOD_NAME, SOCK_DESCR, id);
	if (!res)
	{
		eDynamicSetError(script,EMBER_TYPE_TEXT, "Bad result handle");
		return(-1);
	}
	numRows = msqlNumRows(res);
	eDynamicSetIntValue(script, &script->returnValue, numRows);
	return(0);
}



int doMsqlDataSeek(script, args)
	ember	*script;
	e_stack	*args;
{
	int	pos;
	u_int	id;
	m_result *res;

	id = emIntArg(args);
	pos = emIntArg(args->next);

	res = (m_result *)eDynamicFetchObject(script, MOD_NAME, SOCK_DESCR, id);
	if (!res)
	{
		eDynamicSetError(script,EMBER_TYPE_TEXT, "Bad result handle");
		return(-1);
	}
	msqlDataSeek(res,pos);
	return(0);
}


int doMsqlFetchField(script, args)
	ember	*script;
	e_stack	*args;
{
	u_int	id;
	int	index;
	m_field	*field;
	m_result *res;
	e_value	val;

	id = emIntArg(args);
	res = (m_result *)eDynamicFetchObject(script, MOD_NAME, SOCK_DESCR, id);
	if (!res)
	{
		eDynamicSetError(script,EMBER_TYPE_TEXT, "Bad result handle");
		return(-1);
	}

	field = msqlFetchField(res);
	if (!field)
	{
		eDynamicSetTextValue(script, &script->returnValue, "");
		return(0);
	}
	index = 0;

	eDynamicSetTextValue(script, &val, field->name);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	eDynamicSetTextValue(script, &val, field->table);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	eDynamicSetIntValue(script, &val, field->type);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	eDynamicSetIntValue(script, &val, field->length);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	eDynamicSetIntValue(script, &val, field->flags);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	return(0);
}


int doMsqlFieldSeek(script, args)
	ember	*script;
	e_stack	*args;
{
	int	pos;
	u_int	id;
	m_result *res;

	if (!fields)
	{
		eDynamicSetTextValue(script, &script->returnValue, "");
		return(0);
	}
	pos = emIntArg(args);
	msqlFieldSeek(fields,pos);
	return(0);
}


int doMsqlNumFields(script, args)
	ember	*script;
	e_stack	*args;
{
	int	numFields;
	u_int	id;
	m_result *res;

	id = emIntArg(args);
	res = (m_result *)eDynamicFetchObject(script, MOD_NAME, SOCK_DESCR, id);
	if (!res)
	{
		eDynamicSetTextValue(script, &script->returnValue, "");
		return(0);
	}
	numFields = msqlNumFields(res);
	eDynamicSetIntValue(script, &script->returnValue, numFields);
	return(0);
}



int doMsqlListDBs(script, args)
	ember	*script;
	e_stack	*args;
{
	m_result *res;
	m_row	cur;
	int	index,
		sock;
	e_value	val;

	sock = emIntArg(args);
	res = msqlListDBs(sock);
	if (!res)
	{
		eDynamicSetError(script,EMBER_TYPE_INT,msqlErrMsg);
		eDynamicSetIntValue(script, &script->returnValue, -1);
		return(0);
	}

	index = 0;
	cur = msqlFetchRow(res);
	while(cur)
	{
		eDynamicSetTextValue(script, &val, cur[0]);
		eDynamicSetArrayElement(script,&script->returnValue,&val,index++);
		cur = msqlFetchRow(res);
	}
	msqlFreeResult(res);
	return(0);
}


int doMsqlListTables(script, args)
	ember	*script;
	e_stack	*args;
{
	m_result *res;
	m_row	cur;
	int	index,
		sock;
	e_value	val;

	sock = emIntArg(args);
	res = msqlListTables(sock);
	if (!res)
	{
		eDynamicSetError(script,EMBER_TYPE_INT,msqlErrMsg);
		eDynamicSetIntValue(script, &script->returnValue, -1);
		return(0);
	}

	eDynamicInitArray(script, &script->returnValue);
	index = 0;
	cur = msqlFetchRow(res);
	while(cur)
	{
		eDynamicSetTextValue(script, &val, cur[0]);
		eDynamicSetArrayElement(script,&script->returnValue,&val,index);
		index++;
		cur = msqlFetchRow(res);
	}
	msqlFreeResult(res);
	return(0);
}


static 	m_result *fields = NULL;

int doMsqlInitFieldList(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*table;
	int	sock;

	sock = emIntArg(args);
	table = emTextArg(args->next);
	if (fields)
		msqlFreeResult(fields);
	fields = msqlListFields(sock,table);
	if (!fields)
	{
		eDynamicSetIntValue(script, &script->returnValue, -1);
	}
	else
	{
		eDynamicSetIntValue(script, &script->returnValue, 0);
	}
	return(0);
}


int doMsqlListField(script, args)
	ember	*script;
	e_stack	*args;
{
	m_field	*field;
	int	index;
	e_value	val;

	if (!fields)
	{
		eDynamicSetTextValue(script, &script->returnValue, "");
		return(0);
	}

	field = msqlFetchField(fields);
	if (!field)
	{
		eDynamicSetTextValue(script, &script->returnValue, "");
		return(0);
	}

	index = 0;
	eDynamicSetTextValue(script, &val, field->name);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	eDynamicSetTextValue(script, &val, field->table);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	eDynamicSetIntValue(script, &val, field->type);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	eDynamicSetIntValue(script, &val, field->length);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	eDynamicSetIntValue(script, &val, field->flags);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	return(0);
}



int doMsqlEncode(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*str,
		*new,
		*cp1,
		*cp2;

	str = emTextArg(args);
	new = (char *)malloc(2 * strlen(str));
	cp1 = str;
	cp2 = new;
	while(*cp1)
	{
		if (*cp1 == '\\')
		{
			*cp2++ = *cp1++;
		}
		else
		{
			if (*cp1=='\'' || *cp1=='[' || *cp1==']' || 
				*cp1=='^' || *cp1=='$' )
			{
				*cp2++ = '\\';
			}
		}
		*cp2++ = *cp1++;
	}
	*cp2 = 0;
	eDynamicSetTextValue(script, &script->returnValue, new);
	free(new);
	return(0);
}



int doMsqlDateToUnixTime(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*date;
	time_t	res;

	date = emTextArg(args);
	res = msqlDateToUnixTime(date);
	eDynamicSetIntValue(script, &script->returnValue, res);
	return(0);
}


int doMsqlUnixTimeToDate(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*date;
	time_t	time;

	time = emIntArg(args);
	date = (char *)msqlUnixTimeToDate(time);
	eDynamicSetTextValue(script, &script->returnValue, date);
	return(0);
}



int doMsqlTimeToUnixTime(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*time;
	time_t	res;

	time = emTextArg(args);
	res = msqlTimeToUnixTime(time);
	eDynamicSetIntValue(script, &script->returnValue, res);
	return(0);
}


int doMsqlUnixTimeToTime(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*time;
	time_t	utime;

	utime = emIntArg(args);
	time = (char *)msqlUnixTimeToTime(utime);
	eDynamicSetTextValue(script, &script->returnValue, time);
	return(0);
}


int doMsqlSumTimes(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*time1,
		*time2,
		*result;

	time1 = emTextArg(args);
	time2 = emTextArg(args->next);
	result = (char *)msqlSumTimes(time1,time2);
	eDynamicSetTextValue(script, &script->returnValue, result);
	return(0);
}


int doMsqlDateOffset(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*date,
		*result;
	int	dOff,
		mOff,
		yOff;

	date = emTextArg(args);
	dOff = emIntArg(args->next);
	mOff = emIntArg(args->next->next);
	yOff = emIntArg(args->next->next->next);
	result = (char *)msqlDateOffset(date,dOff, mOff, yOff);
	eDynamicSetTextValue(script, &script->returnValue, result);
	return(0);
}


int doMsqlDiffTimes(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*time1,
		*time2,
		*result;

	time1 = emTextArg(args);
	time2 = emTextArg(args->next);
	result = (char *)msqlDiffTimes(time1,time2);
	eDynamicSetTextValue(script, &script->returnValue, result);
	return(0);
}


int doMsqlDiffDates(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*date1,
		*date2;
	int	result;

	date1 = emTextArg(args);
	date2 = emTextArg(args->next);
	result = msqlDiffDates(date1,date2);
	eDynamicSetIntValue(script, &script->returnValue, result);
	return(0);
}


int doMsqlLoadConfigFile(script, args)
	ember	*script;
	e_stack *args;
{
	char	*file;
	int	result;

	file = emTextArg(args);
	result = msqlLoadConfigFile(file);
	eDynamicSetIntValue(script, &script->returnValue, result);
	return(0);
}

int init_mod(script, modPtr)
        ember   *script;  
        void    *modPtr;
{
        if (eDynamicAddModule(script, modPtr, msql_externs) < 0)
        {
                return(-1);
        }
        else
        {
                return(0);
        }
}

==> mod_msql-1.2.tar.gz: mod_msql-1.2/src/mod_msql.c <==
/*	mod_msql.c	- 
**
**
** Copyright (c) 1995-2001  Hughes Technologies Pty Ltd.
**
** Permission to use, copy, and distribute for non-commercial purposes,
** is hereby granted without fee, providing that the above copyright
** notice appear in all copies and that both the copyright notice and this
** permission notice appear in supporting documentation.
**
** The software may be modified for your own purposes, but modified versions
** may not be distributed.
**
** This software is provided "as is" without any expressed or implied warranty.
**
*/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <time.h>
#include "ember.h"
#include "msql.h"
#include "mod_msql.h"


#define	MOD_NAME	"mod_msql"
#define SOCK_DESCR	"sock"

static 	m_result *fieldListResult = NULL;

int doMsqlConnect(script, args)
	ember	*script;
	e_stack	*args;
{
	int	sock;
	char	*host = NULL;

	if (args)
	{
		if (emArgType(args) != EMBER_TYPE_TEXT)
		{
			eDynamicSetError(script, EMBER_TYPE_INT, 
				"Invalid parameter");
			return(0);
		}
		host = emTextArg(args);
		if (args->next)
		{
			eDynamicSetError(script, EMBER_TYPE_INT, 
				"Too many arguments");
			return(0);
		}
	}
	if ((sock = msqlConnect(host)) < 0)
	{
		eDynamicSetError(script,EMBER_TYPE_INT,msqlErrMsg);
		return(0);
	}
	eDynamicSetIntValue(script, &script->returnValue, sock);
	return(0);
}

int doMsqlClose(script, args)
	ember	*script;
	e_stack	*args;
{
	int	sock;

	sock = emIntArg(args);
	msqlClose(sock);
	eDynamicSetIntValue(script, &script->returnValue, 0);
	return(0);
}



int doMsqlSelectDB(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*dbName;
	int	sock;


	sock = emIntArg(args);
	dbName = emTextArg(args->next);
	if (msqlSelectDB(sock,dbName) < 0)
	{
		eDynamicSetError(script,EMBER_TYPE_INT,msqlErrMsg);
		eDynamicSetIntValue(script, &script->returnValue, -1);
	}
	else
	{
		eDynamicSetIntValue(script, &script->returnValue, 0);
	}
	return(0);
}




int doMsqlQuery(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*query;
	int	res,
		sock;


	sock = emIntArg(args);
	query = emTextArg(args->next);
	res = msqlQuery(sock,query);
	if (res < 0)
	{
		eDynamicSetError(script,EMBER_TYPE_INT,msqlErrMsg);
	}
	eDynamicSetIntValue(script, &script->returnValue, res);
	return(0);
}


int doMsqlStoreResult(script, args)
	ember	*script;
	e_stack	*args;
{
	m_result	*res;
	u_int		id;

	res = msqlStoreResult();
	id = eDynamicStoreObject(script, MOD_NAME, SOCK_DESCR, (void *)res);
	eDynamicSetIntValue(script, &script->returnValue, id);
	return(0);
}


int doMsqlFreeResult(script, args)
	ember	*script;
	e_stack	*args;
{
	int		id;
	m_result	*res;

	id = emIntArg(args);
	res = (m_result *)eDynamicDeleteObject(script, MOD_NAME, SOCK_DESCR, id);
	if (res)
	{
		msqlFreeResult(res);
	}
	return(0);
}



int doMsqlFetchRow(script, args)
	ember	*script;
	e_stack	*args;
{
	u_int	id;
	int	numFields,
		index;
	m_row	row;
	m_result *res;
	e_value	val;

	id = emIntArg(args);
	res = (m_result *)eDynamicFetchObject(script, MOD_NAME,SOCK_DESCR, id);
	if (!res)
	{
		eDynamicSetError(script,EMBER_TYPE_TEXT, "Bad result handle");
		return(-1);
	}

	numFields = msqlNumFields(res);
	row = msqlFetchRow(res);
	if (!row)
	{
		eDynamicSetTextValue(script, &script->returnValue, "");
		return(0);
	}
	index = 0;
	while (index < numFields)
	{
		if (row[index])
			eDynamicSetTextValue(script,&val,row[index]);
		else
			eDynamicSetTextValue(script,&val,"");
		eDynamicSetArrayElement(script, &script->returnValue, &val, index);
		index++;
	}
	return(0);
}

int doMsqlNumRows(script, args)
	ember	*script;
	e_stack	*args;
{
	int	numRows;
	u_int	id;
	m_result *res;

	id = emIntArg(args);
	res = (m_result *)eDynamicFetchObject(script, MOD_NAME, SOCK_DESCR, id);
	if (!res)
	{
		eDynamicSetError(script,EMBER_TYPE_TEXT, "Bad result handle");
		return(-1);
	}
	numRows = msqlNumRows(res);
	eDynamicSetIntValue(script, &script->returnValue, numRows);
	return(0);
}



int doMsqlDataSeek(script, args)
	ember	*script;
	e_stack	*args;
{
	int	pos;
	u_int	id;
	m_result *res;

	id = emIntArg(args);
	pos = emIntArg(args->next);

	res = (m_result *)eDynamicFetchObject(script, MOD_NAME, SOCK_DESCR, id);
	if (!res)
	{
		eDynamicSetError(script,EMBER_TYPE_TEXT, "Bad result handle");
		return(-1);
	}
	msqlDataSeek(res,pos);
	return(0);
}


int doMsqlFetchField(script, args)
	ember	*script;
	e_stack	*args;
{
	u_int	id;
	int	index;
	m_field	*field;
	m_result *res;
	e_value	val;

	id = emIntArg(args);
	res = (m_result *)eDynamicFetchObject(script, MOD_NAME, SOCK_DESCR, id);
	if (!res)
	{
		eDynamicSetError(script,EMBER_TYPE_TEXT, "Bad result handle");
		return(-1);
	}

	field = msqlFetchField(res);
	if (!field)
	{
		eDynamicSetTextValue(script, &script->returnValue, "");
		return(0);
	}
	index = 0;

	eDynamicSetTextValue(script, &val, field->name);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	eDynamicSetTextValue(script, &val, field->table);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	eDynamicSetIntValue(script, &val, field->type);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	eDynamicSetIntValue(script, &val, field->length);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	eDynamicSetIntValue(script, &val, field->flags);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	return(0);
}


int doMsqlFieldSeek(script, args)
	ember	*script;
	e_stack	*args;
{
	int	pos;

	if (!fieldListResult)
	{
		eDynamicSetTextValue(script, &script->returnValue, "");
		return(0);
	}
	pos = emIntArg(args);
	msqlFieldSeek(fieldListResult,pos);
	return(0);
}


int doMsqlNumFields(script, args)
	ember	*script;
	e_stack	*args;
{
	int	numFields;
	u_int	id;
	m_result *res;

	id = emIntArg(args);
	res = (m_result *)eDynamicFetchObject(script, MOD_NAME, SOCK_DESCR, id);
	if (!res)
	{
		eDynamicSetTextValue(script, &script->returnValue, "");
		return(0);
	}
	numFields = msqlNumFields(res);
	eDynamicSetIntValue(script, &script->returnValue, numFields);
	return(0);
}



int doMsqlListDBs(script, args)
	ember	*script;
	e_stack	*args;
{
	m_result *res;
	m_row	cur;
	int	index,
		sock;
	e_value	val;

	sock = emIntArg(args);
	res = msqlListDBs(sock);
	if (!res)
	{
		eDynamicSetError(script,EMBER_TYPE_INT,msqlErrMsg);
		eDynamicSetIntValue(script, &script->returnValue, -1);
		return(0);
	}

	index = 0;
	cur = msqlFetchRow(res);
	while(cur)
	{
		eDynamicSetTextValue(script, &val, cur[0]);
		eDynamicSetArrayElement(script,&script->returnValue,&val,index++);
		cur = msqlFetchRow(res);
	}
	msqlFreeResult(res);
	return(0);
}


int doMsqlListTables(script, args)
	ember	*script;
	e_stack	*args;
{
	m_result *res;
	m_row	cur;
	int	index,
		sock;
	e_value	val;

	sock = emIntArg(args);
	res = msqlListTables(sock);
	if (!res)
	{
		eDynamicSetError(script,EMBER_TYPE_INT,msqlErrMsg);
		eDynamicSetIntValue(script, &script->returnValue, -1);
		return(0);
	}

	eDynamicInitArray(script, &script->returnValue);
	index = 0;
	cur = msqlFetchRow(res);
	while(cur)
	{
		eDynamicSetTextValue(script, &val, cur[0]);
		eDynamicSetArrayElement(script,&script->returnValue,&val,index);
		index++;
		cur = msqlFetchRow(res);
	}
	msqlFreeResult(res);
	return(0);
}



int doMsqlInitFieldList(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*table;
	int	sock;

	sock = emIntArg(args);
	table = emTextArg(args->next);
	if (fieldListResult)
		msqlFreeResult(fieldListResult);
	fieldListResult = msqlListFields(sock,table);
	if (!fieldListResult)
	{
		eDynamicSetIntValue(script, &script->returnValue, -1);
	}
	else
	{
		eDynamicSetIntValue(script, &script->returnValue, 0);
	}
	return(0);
}


int doMsqlListField(script, args)
	ember	*script;
	e_stack	*args;
{
	m_field	*field;
	int	index;
	e_value	val;

	if (!fieldListResult)
	{
		eDynamicSetTextValue(script, &script->returnValue, "");
		return(0);
	}

	field = msqlFetchField(fieldListResult);
	if (!field)
	{
		eDynamicSetTextValue(script, &script->returnValue, "");
		return(0);
	}

	index = 0;
	eDynamicSetTextValue(script, &val, field->name);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	eDynamicSetTextValue(script, &val, field->table);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	eDynamicSetIntValue(script, &val, field->type);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	eDynamicSetIntValue(script, &val, field->length);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	eDynamicSetIntValue(script, &val, field->flags);
	eDynamicSetArrayElement(script, &script->returnValue, &val, index++);
	return(0);
}



int doMsqlEncode(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*str,
		*new,
		*cp1,
		*cp2;

	str = emTextArg(args);
	new = (char *)malloc(2 * strlen(str));
	cp1 = str;
	cp2 = new;
	while(*cp1)
	{
		if (*cp1 == '\\')
		{
			*cp2++ = *cp1++;
		}
		else
		{
			if (*cp1=='\'' || *cp1=='[' || *cp1==']' || 
				*cp1=='^' || *cp1=='$' )
			{
				*cp2++ = '\\';
			}
		}
		*cp2++ = *cp1++;
	}
	*cp2 = 0;
	eDynamicSetTextValue(script, &script->returnValue, new);
	free(new);
	return(0);
}



int doMsqlDateToUnixTime(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*date;
	time_t	res;

	date = emTextArg(args);
	res = msqlDateToUnixTime(date);
	eDynamicSetIntValue(script, &script->returnValue, res);
	return(0);
}


int doMsqlUnixTimeToDate(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*date;
	time_t	time;

	time = emIntArg(args);
	date = (char *)msqlUnixTimeToDate(time);
	eDynamicSetTextValue(script, &script->returnValue, date);
	return(0);
}



int doMsqlTimeToUnixTime(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*time;
	time_t	res;

	time = emTextArg(args);
	res = msqlTimeToUnixTime(time);
	eDynamicSetIntValue(script, &script->returnValue, res);
	return(0);
}


int doMsqlUnixTimeToTime(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*time;
	time_t	utime;

	utime = emIntArg(args);
	time = (char *)msqlUnixTimeToTime(utime);
	eDynamicSetTextValue(script, &script->returnValue, time);
	return(0);
}


int doMsqlSumTimes(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*time1,
		*time2,
		*result;

	time1 = emTextArg(args);
	time2 = emTextArg(args->next);
	result = (char *)msqlSumTimes(time1,time2);
	eDynamicSetTextValue(script, &script->returnValue, result);
	return(0);
}


int doMsqlDateOffset(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*date,
		*result;
	int	dOff,
		mOff,
		yOff;

	date = emTextArg(args);
	dOff = emIntArg(args->next);
	mOff = emIntArg(args->next->next);
	yOff = emIntArg(args->next->next->next);
	result = (char *)msqlDateOffset(date,dOff, mOff, yOff);
	eDynamicSetTextValue(script, &script->returnValue, result);
	return(0);
}


int doMsqlDiffTimes(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*time1,
		*time2,
		*result;

	time1 = emTextArg(args);
	time2 = emTextArg(args->next);
	result = (char *)msqlDiffTimes(time1,time2);
	eDynamicSetTextValue(script, &script->returnValue, result);
	return(0);
}


int doMsqlDiffDates(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*date1,
		*date2;
	int	result;

	date1 = emTextArg(args);
	date2 = emTextArg(args->next);
	result = msqlDiffDates(date1,date2);
	eDynamicSetIntValue(script, &script->returnValue, result);
	return(0);
}


int doMsqlLoadConfigFile(script, args)
	ember	*script;
	e_stack *args;
{
	char	*file;
	int	result;

	file = emTextArg(args);
	result = msqlLoadConfigFile(file);
	eDynamicSetIntValue(script, &script->returnValue, result);
	return(0);
}

int init_mod(script, modPtr)
        ember   *script;  
        void    *modPtr;
{
        if (eDynamicAddModule(script, modPtr, msql_externs) < 0)
        {
                return(-1);
        }
        else
        {
                return(0);
        }
}
